%ODE for use with MATLAB solvers
%t is the time corresponding to seconds
%x is a n by n by 4 matrix of chemical concentrations, where the first two dimensions correspond to spatial coordinates and the third corresponds to the four chemical species in our model
%drop_locs is an n by n binary matrix with 1s in each location where a drop is and 0s elsewhere
%params is a vector of terms which are involved in determining reaction rates
%equations is an n by n cell array of functions which calculate the differential equations for the reactions in our model
%light_fns is an n by n cell array of functions which give the light intensity as a function of time

function xdot = coupled_drop_ODE(t, x, drop_locs, params, equations, light_fns)
    %couples drops through inhibitor
    x_size = numel(x);
    x = reshape(x, [size(drop_locs, 1), size(drop_locs, 2), numel(x) / size(drop_locs, 1) / size(drop_locs, 2)]);

    xdot = zeros(size(x));
    
    %diffusion constant
    d = .4 * 10^-3;

    for i = 1:size(x, 1)
        for j = 1:size(x, 2)
            if numel(equations{i, j}) > 0
                f = equations{i, j};
                xdot(i, j, :) = f(t, x(i, j, :), params, light_fns{i, j});
            else
                xdot(i, j, :) = 0;
            end
        end
    end
    
    drop_locs = padarray(drop_locs, [1, 1]);
    diff_counts = circshift(drop_locs, [1, 0]) + circshift(drop_locs, [-1, 0]) + circshift(drop_locs, [0, 1]) + circshift(drop_locs, [0, -1]);
    temp = padarray(squeeze(x(:, :, 2)), [1, 1]);
    temp2 = temp + d * squeeze(circshift(temp, [1, 0]) + circshift(temp, [-1, 0]) + ...
        circshift(temp, [0, 1]) + circshift(temp, [0, -1]) - diff_counts .* temp);
    temp2 = temp2 .* drop_locs;
    dtemp = temp2 - temp;
    
  
    
%     temp = zeros(size(temp));
    xdot(:, :, 2) = xdot(:, :, 2) + dtemp(2:(end - 1), 2:(end - 1));
    xdot = reshape(xdot, [x_size, 1]);
    
end